@@ -3,7 +3,7 @@
  * Programmer(s): Daniel Reynolds @ SMU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2020, Lawrence Livermore National Security
+ * Copyright (c) 2002-2023, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -106,6 +106,7 @@ int main(int argc, char *argv[])
   sunindextype    i;
   realtype        *vecdata;
   double          tol;
+  SUNContext      sunctx;
 
   /* Set up MPI environment */
   fails = MPI_Init(&argc, &argv);
@@ -116,6 +117,11 @@ int main(int argc, char *argv[])
   fails = MPI_Comm_rank(ProbData.comm, &(ProbData.myid));
   if (check_flag(&fails, "MPI_Comm_rank", 1)) return 1;
 
+  if (SUNContext_Create(&ProbData.comm, &sunctx)) {
+    printf("ERROR: SUNContext_Create failed\n");
+    return(-1);
+  }
+
   /* check inputs: local problem size, timing flag */
   if (argc < 6) {
     printf("ERROR: FIVE (5) Inputs required:\n");
@@ -163,22 +169,22 @@ int main(int argc, char *argv[])
 
   /* Create vectors */
   x = N_VNew_Parallel(ProbData.comm, ProbData.Nloc,
-                      ProbData.nprocs * ProbData.Nloc);
+                      ProbData.nprocs * ProbData.Nloc, sunctx);
   if (check_flag(x, "N_VNew_Parallel", 0)) return 1;
   xhat = N_VNew_Parallel(ProbData.comm, ProbData.Nloc,
-                         ProbData.nprocs * ProbData.Nloc);
+                         ProbData.nprocs * ProbData.Nloc, sunctx);
   if (check_flag(xhat, "N_VNew_Parallel", 0)) return 1;
   b = N_VNew_Parallel(ProbData.comm, ProbData.Nloc,
-                      ProbData.nprocs * ProbData.Nloc);
+                      ProbData.nprocs * ProbData.Nloc, sunctx);
   if (check_flag(b, "N_VNew_Parallel", 0)) return 1;
   ProbData.d = N_VNew_Parallel(ProbData.comm, ProbData.Nloc,
-                               ProbData.nprocs * ProbData.Nloc);
+                               ProbData.nprocs * ProbData.Nloc, sunctx);
   if (check_flag(ProbData.d, "N_VNew_Parallel", 0)) return 1;
   ProbData.s1 = N_VNew_Parallel(ProbData.comm, ProbData.Nloc,
-                                ProbData.nprocs * ProbData.Nloc);
+                                ProbData.nprocs * ProbData.Nloc, sunctx);
   if (check_flag(ProbData.s1, "N_VNew_Parallel", 0)) return 1;
   ProbData.s2 = N_VNew_Parallel(ProbData.comm, ProbData.Nloc,
-                                ProbData.nprocs * ProbData.Nloc);
+                                ProbData.nprocs * ProbData.Nloc, sunctx);
   if (check_flag(ProbData.s2, "N_VNew_Parallel", 0)) return 1;
 
   /* Fill xhat vector with uniform random data in [1,2] */
@@ -190,7 +196,7 @@ int main(int argc, char *argv[])
   N_VConst(FIVE, ProbData.d);
 
   /* Create SPFGMR linear solver */
-  LS = SUNLinSol_SPFGMR(x, PREC_RIGHT, maxl);
+  LS = SUNLinSol_SPFGMR(x, SUN_PREC_RIGHT, maxl, sunctx);
   fails += Test_SUNLinSolGetType(LS, SUNLINEARSOLVER_ITERATIVE,
                                  ProbData.myid);
   fails += Test_SUNLinSolGetID(LS, SUNLINEARSOLVER_SPFGMR, ProbData.myid);
@@ -199,6 +205,7 @@ int main(int argc, char *argv[])
                                            PSolve, ProbData.myid);
   fails += Test_SUNLinSolSetScalingVectors(LS, ProbData.s1, ProbData.s2,
                                            ProbData.myid);
+  fails += Test_SUNLinSolSetZeroGuess(LS, ProbData.myid);
   fails += Test_SUNLinSolInitialize(LS, ProbData.myid);
   fails += Test_SUNLinSolSpace(LS, ProbData.myid);
   fails += SUNLinSol_SPFGMRSetGSType(LS, gstype);
@@ -223,9 +230,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNLinSol_SPFGMRSetPrecType(LS, PREC_NONE);
+  fails += SUNLinSol_SPFGMRSetPrecType(LS, SUN_PREC_NONE);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -254,9 +262,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNLinSol_SPFGMRSetPrecType(LS, PREC_RIGHT);
+  fails += SUNLinSol_SPFGMRSetPrecType(LS, SUN_PREC_RIGHT);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -287,9 +296,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNLinSol_SPFGMRSetPrecType(LS, PREC_NONE);
+  fails += SUNLinSol_SPFGMRSetPrecType(LS, SUN_PREC_NONE);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -320,9 +330,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNLinSol_SPFGMRSetPrecType(LS, PREC_RIGHT);
+  fails += SUNLinSol_SPFGMRSetPrecType(LS, SUN_PREC_RIGHT);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -353,9 +364,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNLinSol_SPFGMRSetPrecType(LS, PREC_NONE);
+  fails += SUNLinSol_SPFGMRSetPrecType(LS, SUN_PREC_NONE);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -386,9 +398,10 @@ int main(int argc, char *argv[])
   if (check_flag(&fails, "ATimes", 1)) return 1;
 
   /* Run tests with this setup */
-  fails += SUNLinSol_SPFGMRSetPrecType(LS, PREC_RIGHT);
+  fails += SUNLinSol_SPFGMRSetPrecType(LS, SUN_PREC_RIGHT);
   fails += Test_SUNLinSolSetup(LS, NULL, ProbData.myid);
-  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNTRUE, ProbData.myid);
+  fails += Test_SUNLinSolSolve(LS, NULL, x, b, tol, SUNFALSE, ProbData.myid);
   fails += Test_SUNLinSolLastFlag(LS, ProbData.myid);
   fails += Test_SUNLinSolNumIters(LS, ProbData.myid);
   fails += Test_SUNLinSolResNorm(LS, ProbData.myid);
@@ -413,6 +426,7 @@ int main(int argc, char *argv[])
   N_VDestroy(ProbData.d);
   N_VDestroy(ProbData.s1);
   N_VDestroy(ProbData.s2);
+  SUNContext_Free(&sunctx);
 
   MPI_Finalize();
   return(fails);
@@ -563,14 +577,14 @@ int check_vector(N_Vector X, N_Vector Y, realtype tol)
 
   /* check vector data */
   for(i=0; i<local_problem_size; i++)
-    failure += FNEQ(Xdata[i], Ydata[i], FIVE*tol*SUNRabs(Xdata[i]));
+    failure += SUNRCompareTol(Xdata[i], Ydata[i], tol);
 
   if (failure > ZERO) {
     maxerr = ZERO;
     for(i=0; i < local_problem_size; i++)
       maxerr = SUNMAX(SUNRabs(Xdata[i]-Ydata[i])/SUNRabs(Xdata[i]), maxerr);
     printf("check err failure: maxerr = %"GSYM" (tol = %"GSYM")\n",
-	   maxerr, FIVE*tol);
+	   maxerr, tol);
     return(1);
   }
   else
@@ -579,4 +593,4 @@ int check_vector(N_Vector X, N_Vector Y, realtype tol)
 
 void sync_device()
 {
-}
\ No newline at end of file
+}
